
indexing
	description: "Compact class to get CGI parameters"
	limitations: "No support for escaped characters. Inefficient?"
	compilers: Smalleiffel
	project: "http://www.nenie.org/eiffel/"
	license: "http://www.opensource.org/licenses/mit-license.html"
	revision: "$Id: $"

class
	MINIMAL_CGI_QUERY

creation
	make

feature {NONE} -- Creation

	make is
			-- Do nothing.
		do
		end

feature -- Core operations
	
	has_value (a_name: STRING): BOOLEAN is
			-- Is there a value with that name?
		require
			not_void: a_name /= Void
		local
			start, i: INTEGER
			a_query: STRING
		do
			from
				start := 1
				i := 1
				a_query := query_string
			until
				(i > a_query.count) or Result
			loop
				if is_parameter_separator (a_query.item (i)) then
					start := i + 1
				elseif is_name_value_separator (a_query.item (i)) and (i > start) then
					Result := case_equal (a_name, a_query.substring (start, i - 1))
				end
				i := i + 1
			end
		end

	value (a_name: STRING): STRING is
			-- Value associated with that name?
		require
			not_void: a_name /= Void
			has_value: has_value (a_name)
		local
			start, i: INTEGER
			value_start: INTEGER
			a_query: STRING
		do
			from
				start := 1
				i := 1
				a_query := query_string
			invariant
				has_value: (i > a_query.count) implies (value_start > 0)
			until
				(Result /= Void)
			loop
				if i > a_query.count or else is_parameter_separator (a_query.item (i)) then
					if value_start > 0 then
						!! Result.make (0) -- stop in any case
						if i > value_start then
							Result := a_query.substring (value_start, i - 1)
						end
					elseif i > a_query.count then
						!! Result.make (0) -- paranoia
					else
						start := i + 1
					end
				elseif is_name_value_separator (a_query.item (i)) and (i > start) then
					if case_equal (a_name, a_query.substring (start, i - 1)) then
						value_start := i + 1
					end
				end
				i := i + 1
			end
		ensure
			not_void: Result /= Void
		end

feature -- Operation(s)

	has_true_value (a_name: STRING): BOOLEAN is
			-- Has value equal to 'yes' or similar?
		require
			not_void: a_name /= Void
		local
			a_value: STRING
		do
			if has_value (a_name) then
				a_value := value (a_name)
				Result := case_equal (a_value, "yes") or case_equal (a_value, "true") or case_equal (a_value, "on")
			end
		end

feature {NONE} -- Parsing

	case_equal (a, other: STRING): BOOLEAN is
			-- Case insensitive equality.
		require
			not_void: a /= Void
			other_not_void: other /= Void
		local
			low_a, low_other: STRING
		do
			!! low_a.make (0)
			low_a.copy (a)
			low_a.to_lower

			!! low_other.make (0)
			low_other.copy (other)
			low_other.to_lower
			
			Result := low_a.is_equal (low_other)
		end

feature {NONE} -- Constants
	
	query_string: STRING is
			-- Query string.
		do
			Result := get_environment_variable ("QUERY_STRING")
			if Result = Void then
				Result := ""
			end
		end

	is_name_value_separator (a_char: CHARACTER): BOOLEAN is
			-- Is name=value separator.
		do
			Result := (a_char = '=')
		end

	is_parameter_separator (a_char: CHARACTER): BOOLEAN is
			-- Is parameter separator?
		do
			Result := (a_char = '&') or (a_char = ';')
		end

end

